@@ -1,4 +1,3 @@
-
 package Data::Match;
 
 #########################################################################
@@ -49,7 +48,7 @@ C<REST> matches all remaining elements of an array or hash.
   matches([ 1, 2, 3 ], [ 1, REST() ]); # TRUE
   matches({ 'a'=>1, 'b'=>1 }, { 'b'=>1, REST() => REST() }); # TRUE
 
-C<FIND> searches at all depths for matching subpatterns.
+C<FIND> searches at all depths for matching sub-patterns.
 
   matches([ 1, [ 1, 2 ], 3], FIND(COLLECT('x', [ 1, REST() ])); # is true.
 
@@ -57,20 +56,33 @@ See the test script C<t/t1.t> in the package distribution for more pattern examp
 
 =head1 MATCH COLLECTIONS
 
-When a C<BIND> or C<COLLECT> matches a datum, an entry is collected in C<$result->{BIND}> and C<$result->{COLLECT}>, respectively.  (This might change in the future)
+When a C<BIND> or C<COLLECT> matches a datum, an entry is collected in C<$result-E<gt>{BIND}> and C<$result-E<gt>{COLLECT}>, respectively.  (This might change in the future)
 
-Each entry for the binding name is a hash containing C<'v'>, C<'p'> and C<'ps'> lists.  
-C<'v'> is a list of the value at each match.
-C<'p'> is a list describing where the corresponding match was found based on the root of the searchat each match.
-C<'ps'> is a list of code strings that describes where the match was for each match.
+Each entry for the binding name is a hash containing C<'v'>, C<'p'> and C<'ps'> lists.
 
-=head1 SUBPATTERNS
+=over 4
 
-All patterns can have sub-patterns.  Most patterns match the ANDed results of their sub-patterns and their own behavior, first trying the sub-patterns before attempting to match the intrinsic behavior.  However, C<OR> and C<ANY> match any sub-patterns;
+=item C<'v'>
+
+is a list of the value at each match.
+
+=item C<'p'>
+
+is a list of match paths describing where the corresponding match was found based on the root of the search at each match.  See C<match_path_*>.  C<'p'> is not collected if C<$matchobj-C<gt>{'no_collect_path'}>.
+
+=item C<'ps'>
+
+is a list of code strings (C<match_path_str>) that describes where the match was for each match.  C<'ps'> is collected only if C<$matchobj-C<gt>{'collect_path_str'}>.
+
+=over
+
+=head1 SUB-PATTERNS
+
+All patterns can have sub-patterns.  Most patterns match the AND-ed results of their sub-patterns and their own behavior, first trying the sub-patterns before attempting to match the intrinsic behavior.  However, C<OR> and C<ANY> match any sub-patterns;
 
 For example:
 
-  match([ ['a', 1 ], ['b', 2], ['a', 3] ], EACH(COLLECT('x', ['a', ANY())))) # TRUE
+  match([ ['a', 1 ], ['b', 2], ['a', 3] ], EACH(COLLECT('x', ['a', ANY() ]))) # TRUE
 
 The above pattern means:
 
@@ -80,7 +92,7 @@ For EACH element in the root structure (an array):
 
 =over 2
 
-COLLECT each element, in C<'x'>, that is,
+COLLECT each element, into collection named C<'x'>, that is,
 
 =over 2
 
@@ -102,7 +114,7 @@ Because the second root element (an array) does not start with C<'a'>.  But,
   match( [ ['a', 1 ], ['a', 2], ['a', 3] ], ALL(COLLECT('x', [ 'a', ANY() ])) ) 
   # IS TRUE
 
-The pattern below essentially does a flatten of the nested array:
+The pattern below flattens the nested array into atoms:
 
   match(
     [ 1, 'x', 
@@ -118,7 +130,7 @@ The pattern below essentially does a flatten of the nested array:
     ], 
     FIND(COLLECT('x', EXPR(q{! ref}))), 
     { 'no_collect_path' => 1 }
-  );
+  )->{'COLLECT'}{'x'}{'v'};
 
 C<no_collect_path> causes C<COLLECT> and C<BIND>  to not collect any paths.
 
@@ -132,20 +144,20 @@ For example:
   do {
     my $a = [ 1, 2, 3, 4 ];
     my $p = [ 1, ANY, REST(BIND('s')) ];
-    my $r = matches($a, $p); # 
-    ok($r); # TRUE
+    my $r = matches($a, $p);
+    ok($r);                                           # TRUE
     ok(Compare($r->{'BIND'}{'s'}{'v'}[0], [ 3, 4 ])); # TRUE
-    $p->[0] = 'x';
-    matches($a, [ 1, 2, 'x', 4 ]); # TRUE
+    $r->{'BIND'}{'s'}{'v'}[0][0] = 'x';               # Change match slice
+    matches($a, [ 1, 2, 'x', 4 ]);                    # TRUE
   }
 
-Hash match slices are generate for each key-value pair for a hash matched by C<EACH> and C<ALL>.  Each of these match slices can be matched as a hash with a single key->value pair.
+Hash match slices are generated for each key-value pair for a hash matched by C<EACH> and C<ALL>.  Each of these match slices can be matched as a hash with a single key-value pair.
 
 Match slices are useful for search and replace missions.
 
 =head1 VISITATION ADAPTERS
 
-By default Data::Match is blind to Perl object interfaces.  To instruct Data::Match to not traverse object implementation containers and honor object interfaces you must provide a visitation adaptor.  A visitation adaptor tells Data::Match what to traverse through an object interface and how to keep track of how it got through.
+By default Data::Match is blind to Perl object interfaces.  To instruct Data::Match to not traverse object implementation containers and honor object interfaces you must provide a visitation adapter.  A visitation adapter tells Data::Match how to traverse through an object interface and how to keep track of how it got through.
 
 For example:
 
@@ -173,38 +185,38 @@ For example:
   my $pat = FIND(COLLECT('Foo', ISA('Foo', { 'parent' => $foos->[0], REST() => REST() })));
   $match->match($foos, $pat);
 
-The problem with the above example is: C<FIND> will not honor the interface of class Foo by default and will eventually find a Foo where C<$_->parent eq $foos->[0]> through all the parent and child links in the objects' implementation container.  To force Data::Match to honor an interface (or a subset of an interface) during C<FIND> traversal we create a 'find' adapter sub that will do the right thing.
+The problem with the above example is: C<FIND> will not honor the interface of class Foo by default and will eventually find a Foo where C<$_E<gt>parent eq $foos-E<gt>[0]> through all the parent and child links in the objects' implementation container.  To force Data::Match to honor an interface (or a subset of an interface) during C<FIND> traversal we create a 'find' adapter sub that will do the right thing.
 
   my $opts = {
     'find' => {
        'Foo' => sub {
-	 my ($self, $visitor) = @_;
+	 my ($self, $visitor, $match) = @_;
 
          # Always do 'x'.
          $visitor->($self->x, 'METHOD', 'x');
 
 	 # Optional children traversal.
-	 if ( $match->{'Foo_traverse_children'} ) {
+	 if ( $match->{'Foo_find_children'} ) {
            $visitor->($self->children, 'METHOD', 'children');
 	 }
 
 	 # Optional parent traversal.
-	 if ( $match->{'Foo_traverse_parent'} ) {
+	 if ( $match->{'Foo_find_parent'} ) {
            $visitor->($self->parent, 'METHOD', 'parent');
 	 }
        }
      }
   }
-  my $match = new Data::Match($opts);
-  $match = $match->match($foos, $pat);
+  my $match = new Data::Match($opts, 'Foo_find_children' => 1);
+  $match = $match->execute($foos, $pat);
 
-See C<t/t4.t> for more examples of visitation adaptors.
+See C<t/t4.t> for more examples of visitation adapters.
 
 =head1 DESIGN
 
-Data::Match employs a mostly-functional external interface since this module was inspired by a Lisp tutorial ("The Little Lisper", maybe) I read too many years ago; besides, pattern matching is largely recursively functional.  The optional control hashes and traverse adapter interfaces are better represented by an object interface so implemented a functional veneer over the core object interface.
+Data::Match employs a mostly-functional external interface since this module was inspired by a Lisp tutorial ("The Little Lisper", maybe) I read too many years ago; besides, pattern matching is largely recursively functional.  The optional control hashes and traverse adapter interfaces are better represented by an object interface so I implemented a functional veneer over the core object interface.
 
-Internally, objects are used to represent the pattern primitives because most of the pattern primitives have common behavior.  There are a few design patterns that are particularly applicable in Data::Match: Visitor and Adapter.  Adapter is used to provide the extensiblity for the traversal of blessed structures such that Data::Match can honor the external interfaces of a class and not blindly violate encapsulation.  Visitor is the basis for some of the C<FIND> pattern implementation.  The C<Data::Match::Slice> classes that provide the match slices are probably a Veneer on the array and hash types through the tie meta-behaviors.
+Internally, objects are used to represent the pattern primitives because most of the pattern primitives have common behavior.  There are a few design patterns that are particularly applicable in Data::Match: Visitor and Adapter.  Adapter is used to provide the extensibility for the traversal of blessed structures such that Data::Match can honor the external interfaces of a class and not blindly violate encapsulation.  Visitor is the basis for some of the C<FIND> pattern implementation.  The C<Data::Match::Slice> classes that provide the match slices are probably a Veneer on the array and hash types through the tie meta-behaviors.
 
 =head1 CAVEATS
 
@@ -212,7 +224,7 @@ Internally, objects are used to represent the pattern primitives because most of
 
 =item *
 
-Does not have regex-like operators like '?', '*', '+'.  
+Does not have regexp-like operators like '?', '*', '+'.  
 
 =item *
 
@@ -220,17 +232,25 @@ Should probably have more interfaces with Data::DRef and Data::Walker.
 
 =item *
 
-The visitor adaptors do not use C<UNIVERSAL::isa> to search for the adapter is uses C<ref>.
+The visitor adapters do not use C<UNIVERSAL::isa> to search for the adapter; it uses C<ref>.  This will be fixed in a future release.
 
 =item *
 
-Since hash keys do not retain blessedness (what was Larry thinking?) it is difficult to have patterns match keys without resorting to some bizarre regex instead of using C<isa>. 
+Since hash keys do not retain blessedness (what was Larry thinking?) it is difficult to have patterns match keys without resorting to some bizarre regexp instead of using C<isa>. 
+
+=item *
+
+C<match_path_set> and C<match_path_ref> do not work through C<'METHOD'> path boundaries.  This will be fixed in a future release.
+
+=item *
+
+C<BIND> and C<COLLECT> need scoping operators for deeply collected patterns.
 
 =back
 
 =head1 STATUS
 
-If you find this to be useful please contact the author.  This is alpha software; all APIs, semantics and behavors are subject to change.
+If you find this to be useful please contact the author.  This is alpha software; all APIs, semantics and behaviors are subject to change.
 
 =head1 INTERFACE
 
@@ -245,8 +265,8 @@ This section describes the external interface of this module.
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
-our $REVISION = do { my @r = (q$Revision: 1.8 $ =~ /\d+/g); sprintf "%d." . "%02d" x $#r, @r };
+our $VERSION = '0.05';
+our $REVISION = do { my @r = (q$Revision: 1.11 $ =~ /\d+/g); sprintf "%d." . "%02d" x $#r, @r };
 
 our $PACKAGE = __PACKAGE__;
 
@@ -267,6 +287,7 @@ our @autoload_pat =
      ISA REF 
      DEPTH 
      REST 
+     RANG STAR PLUS QUES
      EACH 
      ALL 
      FIND 
@@ -281,6 +302,9 @@ our %EXPORT_TAGS = (
 		     'pat'  => \@export_pat,
 		     );
 
+use String::Escape qw(printable);
+use Sort::Topological qw(:all);
+
 use Data::Dumper;
 use Data::Compare;
 use Carp qw(confess);
@@ -372,66 +396,59 @@ sub _self_or_instance
 # Low-level container match traversals.
 #
 
+sub _match_ARRAY_REST($$$$$)
+{
+  my ($self, $x, $p, $x_i, $p_i) = @_;
+
+  my $match = 1;
+
+ ARRAY:
+  {
+    # Each element must match.
+    while ( $$p_i < @$p ) {
+      # [ 'x', 'y', REST ] matches [ 'x', 'y', 'z', '1', '2', '3' ]
+      # Where SUBPAT in REST(SUBPAT) is bound to [ 'z', '1', '2', '3' ].
+      if ( ! $self->{'disable_patterns'} && 
+	 UNIVERSAL::isa($p->[$$p_i], 'Data::Match::Pattern::REST') ) {
+
+	# Match REST's subpatterns against the REST slice.
+	$match &&= $p->[$$p_i]->_match_REST_ARRAY($x, $p, $self, $x_i, $p_i);
+      } else {
+	# Match each element of $x against each element $p.
+	$self->_match_path_push('ARRAY', $$x_i);
+	
+	$match = $$x_i < @$x && $self->_match($x->[$$x_i], $p->[$$p_i]);
+	
+	$self->_match_path_pop;
+      }
+
+      last ARRAY unless $match;
+
+      ++ $$x_i;
+      ++ $$p_i;
+    }
+    
+    # Make sure lengths are same.
+    $match &&= $$p_i == @$p && $$x_i == @$x;
+  }
+
+  $match;
+}
+
+
 #=head2 _match_ARRAY
 #
 #Internal recursive match routine.  Assumes $matchobj is initialized.
 #
 #=cut
-sub _match_ARRAY
+sub _match_ARRAY($$$)
 {
   my ($self, $x, $p) = @_;
 
-  my $match;
-
-  ++ $self->{'depth'};
-  push(@{$self->{'path'}}, 'ARRAY');
+  my $x_i = 0;
+  my $p_i = 0;
   
- ARRAY:
-  {
-    $match = 1;
-    
-    # Each element must match.
-    for my $i ( 0 .. $#$p ) {
-      # [ 'x', 'y', REST ] matches [ 'x', 'y', 'z', '1', '2', '3' ]
-      # Where SUBPAT in REST(SUBPAT) is bound to [ 'z', '1', '2', '3' ].
-      if ( ! $self->{'disable_patterns'} && 
-	 UNIVERSAL::isa($p->[$i], 'Data::Match::Pattern::REST') ) {
-	push(@{$self->{'path'}}, [$i, scalar @$x]);
-
-	# Create an new array slice to match the rest of the array.
-	# The Slice::Array object will forward changes to
-	# the real array.
-	my $slice;
-	if ( 1 ) {
-	  $slice = Data::Match::Slice::Array->new($x, $i, scalar @$x);
-	} else {
-	  $slice = [ @{$x}[$i .. $#$x] ];
-	}
-
-	# Match REST's subpatterns against the REST slice.
-	$match &&= $p->[$i]->_match_rest_ARRAY($slice, $self);
-	
-	pop(@{$self->{'path'}});
-
-	last ARRAY;
-      }
-      
-      push(@{$self->{'path'}}, $i);
-      
-      $match = $i < @$x && $self->_match($x->[$i], $p->[$i]);
-      
-      pop(@{$self->{'path'}});
-      
-      last ARRAY unless $match;
-    }
-    
-    $match &&= @$p == @$x;
-  }
-
-  -- $self->{'depth'};
-  pop(@{$self->{'path'}});
-
-  $match;
+  $self->_match_ARRAY_REST($x, $p, \$x_i, \$p_i);
 }
 
 
@@ -441,19 +458,16 @@ sub _match_ARRAY
 #Internal recursive match routine.  Assumes $matchobj is initialized.
 #
 #=cut
-sub _match_HASH
+sub _match_HASH($$$)
 {
   my ($self, $x, $p) = @_;
 
-  my $match = 0;
+  # $DB::single = 1;
+
+  my $match = 1;
 
-  ++ $self->{'depth'};
-  push(@{$self->{'path'}}, 'HASH');
-	  
  HASH:
   {
-    $match = 1;
-    
     my $rest_pat;
     my $any_key = 0;
     
@@ -470,73 +484,54 @@ sub _match_HASH
 	  my $matches = 0;
 	  
 	  for my $xk ( keys %$x ) {
-	    push(@{$self->{'path'}}, $xk);
+	    $self->_match_path_push('HASH', $xk);
 
 	    ++ $matched_keys{$xk};
 	    ++ $matches if $self->_match($x->{$xk}, $p->{$k});
-	    
-	    pop(@{$self->{'path'}});
-	    
-	    last unless $match;
+	    	    
+	    $self->_match_path_pop;
 	  }
 	  
 	  # Must have at least one match.
 	  # { ANY => 'x' } does not match { }.
 	  $match &&= $matches;
 	}
-	next;
       }
       
       # Rest in a pattern causes the rest to match.
-      if ( ! $self->{'disable_patterns'} && 
+      elsif ( ! $self->{'disable_patterns'} && 
 	   ! $rest_pat &&
 	 UNIVERSAL::isa($p->{$k}, 'Data::Match::Pattern::REST')
 	   ) {
 	$rest_pat = $p->{$k};
-	next;
       }
-      
-      # If the key does not exist in pattern, no match.
-      push(@{$self->{'path'}}, $k);
-      
-      ++ $matched_keys{$k};
-      $match &&= exists $x->{$k} && $self->_match($x->{$k}, $p->{$k});
-      
-      pop(@{$self->{'path'}});
-      
+
+      else {
+	# Match the $x value for $k with the pattern value for $k.
+	$self->_match_path_push('HASH', $k);
+	
+	# If the key does not exist in pattern, no match.
+	++ $matched_keys{$k};
+	$match &&= exists $x->{$k} && $self->_match($x->{$k}, $p->{$k});
+	
+	$self->_match_path_pop;
+      }
+
       last HASH unless $match;
     }
     
     # Handle REST pattern's subpatterns.
     if ( $rest_pat ) {
-      my @rest_keys = grep(! exists $matched_keys{$_}, keys %$x);
-
-      # Create a temporary hash slice containing
-      # the values from $x for all the unmatched keys.
-      my $slice;
-      if ( 1 ) {
-	$slice = Data::Match::Slice::Hash->new($x, \@rest_keys);
-      } else {
-	$slice = { };
-	@{$slice}{@rest_keys} = @{$x}{@rest_keys};
-      }
-
-      # See match_path_str().
-      push(@{$self->{'path'}}, \@rest_keys);
-
-      #$DB::single = 1;
-      $match &&= $rest_pat->_match_rest_HASH($slice, $self);
-
-      pop(@{$self->{'path'}});
+      $match &&= $rest_pat->_match_REST_HASH($x, $p, $self, 
+					     # What keys in $x have not been matched against?
+					     [ grep(! exists $matched_keys{$_}, keys %$x) ]
+					     );
     } else {
       # Make sure they are the same length.
       $match &&= (scalar values %$p) == (scalar values %$x) unless $any_key;
     }
   }
 
-  -- $self->{'depth'};
-  pop(@{$self->{'path'}});
-
   $match;
 }
 
@@ -546,19 +541,15 @@ sub _match_HASH
 #Internal recursive match routine.  Assumes $matchobj is initialized.
 #
 #=cut
-sub _match_SCALAR
+sub _match_SCALAR($$$)
 {
   my ($self, $x, $p) = @_;
 
-  ++ $self->{'depth'};
-  push(@{$self->{'path'}}, 'SCALAR');
-	  
-  push(@{$self->{'path'}}, undef); # 'emacs
-  my $match = $self->_match($$x, $$p);
-  pop(@{$self->{'path'}});
+  $self->_match_path_push('SCALAR', undef);
 
-  -- $self->{'depth'};
-  pop(@{$self->{'path'}});
+  my $match = $self->_match($$x, $$p);
+
+  $self->_match_path_pop;
 
   $match;
 };
@@ -570,7 +561,7 @@ sub _match_SCALAR
 #Internal recursive match routine.  Assumes $self is initialized.
 #
 #=cut
-sub _match_path_push
+sub _match_path_push($$$)
 {
   my $self = shift;
   ++ $self->{'depth'};
@@ -587,9 +578,10 @@ sub _match_path_pop
 {
   my $self = shift;
   # $DB::single = 1;
+  confess "too many _match_path_pop" unless $self->{'depth'} > 0;
+  confess "corrupted path" unless (@{$self->{'path'}} & 1) == 0;
   splice(@{$self->{'path'}}, -2);
   -- $self->{'depth'};
-  confess "too many _match_path_pop" if $self->{'depth'} < 0;
 }
 
 
@@ -685,11 +677,11 @@ our %match_opts
      );
 
 
-=head2 _match_pre
-
-Initialize the match object before pattern traversal.
-
-=cut
+#=head2 _match_pre
+#
+#Initialize the match object before pattern traversal.
+#
+#=cut
 sub _match_pre
 {
   my ($self, $x, $p, $opts) = @_;
@@ -711,11 +703,11 @@ sub _match_pre
 }
 
 
-=head2 _match_post
-
-Initialize the match object before pattern traversal.
-
-=cut
+#=head2 _match_post
+#
+#Initialize the match object before pattern traversal.
+#
+#=cut
 sub _match_post
 {
   my ($self, $x, $p) = @_;
@@ -726,7 +718,7 @@ sub _match_post
   {
     no warnings;
 
-    confess "Expected results->{depth} == 0" unless $self->{'depth'} == 0;
+    confess "Expected results->{depth} == 0, found $self->{depth}" unless $self->{'depth'} == 0;
     confess "Expected results->{path} eq [ ]" unless ! @{$self->{'path'}};
     confess "Expected results->{root} eq root" unless $self->{'root'} eq $x;
     confess "Expected results->{pattern} eq pattern" unless $self->{'pattern'} eq $p;
@@ -754,7 +746,7 @@ sub execute
   $self->_match_post($x, $p);
 
   # Return results.
-  if ( wantsarray ) {
+  if ( wantarray ) {
     return ($matches, $self);
   } else {
     return $matches ? $self : undef;
@@ -764,10 +756,12 @@ sub execute
 
 =head2 match
 
+   use Data::Match qw(match);
    match($thing, $pattern, @opts)
 
-it same as:
+is equivalent to:
 
+   use Data::Match;
    Data::Match->new(@opts)->execute($thing, $pattern);
 
 =cut
@@ -795,22 +789,99 @@ sub matches
 
 
 
+#=head2 _match_state_save
+#
+#
+#=cut
+sub _match_state_save
+{
+  my ($self) = @_;
+  
+  my $state = { };
+
+  for my $x ( $self->{'_COLLECT'}, $self->{'_BIND'} ) {
+    my $c = $self->{$x};
+    next unless $c;
+    my $s = $state->{$x} = { };
+    for my $k ( keys %$c ) {
+      @{$s->{$k}{'v'}}       = $c->{$k}{'v'}   ? @{$c->{$k}{'v'}} : () ;
+      @{$s->{$x}{$k}{'p'}}   = $c->{$k}{'p'}   ? @{$c->{$k}{'p'}} : () ;
+      @{$s->{$x}{$k}{'ps'}}  = $c->{$k}{'ps'}  ? @{$c->{$k}{'ps'}} : () ;
+      @{$s->{$x}{$k}{'pdr'}} = $c->{$k}{'pdr'} ? @{$c->{$k}{'pdr'}} : () ;
+    }
+  }
+
+  $state;
+}
+
+
+#=head2 _match_state_restore
+#
+#
+#=cut
+sub _match_state_restore
+{
+  my ($self, $state) = @_;
+
+  for my $x ( $self->{'_COLLECT'}, $self->{'_BIND'} ) {
+    my $c = $self->{$x};
+    next unless $c;
+    my $s = $state->{$x};
+    for my $k ( keys %$c ) {
+      if ( ! $s->{$k} ) {
+	undef $c->{$k};
+	next;
+      }
+      @{$c->{$k}{'v'}}       = $s->{$k}{'v'}   ? @{$s->{$k}{'v'}} : () ;
+      @{$c->{$x}{$k}{'p'}}   = $s->{$k}{'p'}   ? @{$s->{$k}{'p'}} : () ;
+      @{$c->{$x}{$k}{'ps'}}  = $s->{$k}{'ps'}  ? @{$s->{$k}{'ps'}} : () ;
+      @{$c->{$x}{$k}{'pdr'}} = $s->{$k}{'pdr'} ? @{$s->{$k}{'pdr'}} : () ;      
+    }
+  }
+  $self;
+}
+
 ##################################################
 # Path support
 #
 
 
+# String::Escape::printable does not handle '$' and '@' interpolations 
+# in a qq{} context correctly.
+sub qinterp
+{
+  
+  my $x = shift;
+  $x =~ s/([\$\@])/\\$1/sgo;
+  $x;
+}
+
+
+# qprintable is conditional about putting '"' around strings
+# printable is not conditional, so wrap it and throw in a join.
+sub qqquote
+{
+  join(',', map('"' . qinterp(printable($_)) . '"', @_));
+}
+
+
+
 =head2 match_path_str
 
 Returns a perl expression that will generate code to point to the element of the path.
 
+  $matchobj->match_path_str($path, $str);
+
+C<$str> defaults to C<'$_'>.
+
 =cut
 sub match_path_str
 {
-  my ($path, $str, $matchobj) = @_;
+  my ($matchobj, $path, $str) = @_;
 
-  $str ||= '$_';
+  $str = '$_' unless defined $str;
 
+  # $DB::single = ! ref $path;
   my @path = @$path;
 
   while ( @path ) {
@@ -828,10 +899,10 @@ sub match_path_str
     elsif ( $ref eq 'HASH' ) {
       if ( ref($ind) eq 'ARRAY' ) {
 	# Create a temporary hash slice.
-	my $elems = join(',', map('"' . quotemeta($_) . '"', sort @$ind));
+	my $elems = qqquote(sort @$ind);
 	$str = "(Data::Match::Slice::Hash->new($str,[$elems]))";
       } else {
-	$ind = '"'. quotemeta($ind) . '"';
+	$ind = qqquote($ind);
 	$str .= "->{$ind}";
       }
     }
@@ -844,7 +915,7 @@ sub match_path_str
 	my @args = @$ind;
 	my $method = shift @args;
 	
-	$str = $str . "->$method(" . join(',', map('"'. quotemeta($_) . '"', @args)) . ')';
+	$str = $str . "->$method(" . qqquote(@args) . ')';
       } else {
 	$str = $str . "->$ind()";
       }
@@ -863,13 +934,19 @@ sub match_path_str
 
 Returns a string suitable for Data::DRef.
 
+  $matchobj->match_path_DRef_path($path, $str, $sep);
+
+C<$str> is used as a prefix for the Data::DRef path.
+C<$str> defaults to C<''>;
+C<$sep> defaults to C<$Data::DRef::Separator> or C<'.'>;
+
 =cut
 sub match_path_DRef_path
 {
-  my ($path, $str, $matchobj, $sep) = @_;
+  my ($matchobj, $path, $str, $sep) = @_;
 
-  $str ||= '';
-  $sep ||= $Data::DRef::sep || '.';
+  $str = '' unless defined $str;
+  $sep = ($Data::DRef::Separator || '.') unless defined $sep;
 
   my @path = @$path;
 
@@ -917,23 +994,31 @@ sub match_path_DRef_path
 
 Returns the value pointing to the location for the match path in the root.
 
+  $matchobj->match_path_get($path, $root);
+
+C<$root> defaults to C<$matchobj-C<gt>{'root'}>;
+
+Example:
+
   my $results = matches($thing, FIND(BIND('x', [ 'x', REST ])));
-  my $x = match_path_get($thing, $results->{'BIND'}{'x'}{'p'}[0]);
+  my $x = $results->match_path_get($thing, $results->{'BIND'}{'x'}{'p'}[0]);
 
 The above example returns the first array that begins with C<'x'>.
 
 =cut
 sub match_path_get
 {
-  my ($path, $root, $results) = @_;
+  my ($results, $path, $root) = @_;
 
-  my $ps = match_path_str($path, '$_[0]', $results);
+  my $ps = $results->match_path_str($path, '$_[0]');
 
   # warn "ps = $ps" if ( 1 || $ps =~ /,/ );
 
   my $pfunc = eval "sub { $ps; }";
   die "$@: $ps" if $@;
 
+  $root = $results->{'root'} if ! defined $root;
+
   $pfunc->($root);
 }
 
@@ -943,23 +1028,31 @@ sub match_path_get
 
 Returns the value pointing to the location for the match path in the root.
 
+  $matchobj->match_path_set($path, $value, $root);
+
+C<$root> defaults to C<$matchobj-C<gt>{'root'}>;
+
+Example:
+
   my $results = matches($thing, FIND(BIND('x', [ 'x', REST ])));
-  match_path_set($thing, $results->{'BIND'}{'x'}{'p'}[0], 'y');
+  $results->match_path_set($thing, $results->{'BIND'}{'x'}{'p'}[0], 'y');
 
 The above example replaces the first array found that starts with 'x' with 'y';
 
 =cut
 sub match_path_set
 {
-  my ($path, $root, $value, $results) = @_;
+  my ($results, $path, $value, $root) = @_;
 
-  my $ps = match_path_str($path, '$_[0]', $results);
+  my $ps = $results->match_path_str($path, '$_[0]');
 
   # warn "ps = $ps" if ( 1 || $ps =~ /,/ );
 
   my $pfunc = eval "sub { $ps = \$_[1]; }";
   die "$@: $ps" if $@;
 
+  $root = $results->{'root'} if ! defined $root;
+
   $pfunc->($root, $value);
 }
 
@@ -968,8 +1061,14 @@ sub match_path_set
 
 Returns a scalar ref pointing to the location for the match path in the root.
 
+  $matchobj->match_path_ref($path, $root);
+
+C<$root> defaults to C<$matchobj-C<gt>{'root'}>;
+
+Example:
+
   my $results = matches($thing, FIND(BIND('x', [ 'x', REST ])));
-  my $ref = match_path_ref($thing, $results->{'BIND'}{'x'}{'p'}[0]);
+  my $ref = $results->match_path_ref($thing, $results->{'BIND'}{'x'}{'p'}[0]);
   $$ref = 'y';
 
 The above example replaces the first array that starts with 'x' with 'y';
@@ -977,15 +1076,17 @@ The above example replaces the first array that starts with 'x' with 'y';
 =cut
 sub match_path_ref
 {
-  my ($path, $root, $results) = @_;
+  my ($results, $path, $root) = @_;
 
-  my $ps = match_path_str($path, '$_[0]', $results);
+  my $ps = $results->match_path_str($path, '$_[0]');
 
   # warn "ps = $ps" if ( 1 || $ps =~ /,/ );
 
   my $pfunc = eval "sub { \\{$ps}; }";
   die "$@: $ps" if $@;
 
+  $root = $results->{'root'} if ! defined $root;
+
   $pfunc->($root);
 }
 
@@ -1120,10 +1221,10 @@ sub _collect
   push(@{$binding->{'p'}}, $path) 
     unless $matchobj->{'no_collect_path'};
 
-  push(@{$binding->{'ps'}}, Data::Match::match_path_str($path, undef, $matchobj)) 
+  push(@{$binding->{'ps'}}, $matchobj->match_path_str($path)) 
     if ( $matchobj->{'collect_path_str'} );
 
-  push(@{$binding->{'pdr'}}, Data::Match::match_path_DRef_path($path, undef, $matchobj)) 
+  push(@{$binding->{'pdr'}}, $matchobj->match_path_DRef_path($path)) 
     if ( $matchobj->{'collect_path_DRef'} );
 }
 
@@ -1357,19 +1458,188 @@ sub match
 }
 
 
-sub _match_rest_ARRAY
+sub _match_REST_ARRAY($$$$$$)
 {
-  my ($self, $x, $matchobj) = @_;
+  my ($self, $x, $p, $matchobj, $x_i, $p_i) = @_;
 
-  ref($x) && $self->match_and($x, $matchobj);
+  my $match;
+
+  $matchobj->_match_path_push('ARRAY', [$$x_i, scalar @$x]);
+  
+  # Create an new array slice to match the rest of the array.
+  # The Slice::Array object will forward changes to
+  # the real array.
+  my $slice = Data::Match::Slice::Array->new($x, $$x_i, scalar @$x);
+
+  $match = ref($x) && $self->match_and($slice, $matchobj);
+
+  $matchobj->_match_path_pop;
+
+  # Slurp up remaining $x and $p.
+  $$x_i = $#$x;
+  $$p_i = $#$p;
+
+  $match;
 }
 
 
-sub _match_rest_HASH
+sub _match_REST_HASH
+{
+  my ($self, $x, $p, $matchobj, $rest_keys) = @_;
+
+  $matchobj->_match_path_push('HASH', $rest_keys);
+
+  # Create a temporary hash slice containing
+  # the values from $x for all the unmatched keys.
+  my $slice = Data::Match::Slice::Hash->new($x, $rest_keys);
+
+  #$DB::single = 1;
+  my $match = $self->match_and($slice, $matchobj);
+  
+  $matchobj->_match_path_pop;
+
+  $match;
+}
+
+
+##################################################
+
+
+package Data::Match::Pattern::RANG;
+
+our @ISA = qw(Data::Match::Pattern::REST);
+
+
+use Carp qw(confess);
+
+
+sub subpattern_offset { 2; };
+
+
+sub initialize
+{
+  my $self = shift;
+
+  $self->[0] = 0 unless defined $self->[0];
+
+  $self;
+}
+
+
+sub _match_REST_ARRAY
+{
+  my ($self, $x, $p, $matchobj, $x_i, $p_i) = @_;
+
+  # $DB::single = 1;
+
+  my $count = 0;
+
+  my ($match_sub, $match_rest);
+  my $rest_saved_state;
+
+  my $matched_rest;
+
+  # Loop for until entire array is eaten,
+  TRY:
+  while ( 1 ) {
+    # Save the match state for rollback after failure.
+    my $saved_state = $matchobj->_match_state_save;
+
+    # Try to match the subpattern.
+    {
+      my $sub_x_i = $$x_i;
+      my $sub_p_i = $self->subpattern_offset;
+      $match_sub = $matchobj->_match_ARRAY_REST($x, $self, \$sub_x_i, \$sub_p_i);
+            
+      if ( $match_sub ) {
+	$$x_i = $sub_x_i;
+      } else {
+	# Restore match state if failed.
+	$matchobj->_match_state_restore($saved_state);
+      }
+    }
+
+    # Try to match rest of pattern.
+    $saved_state = $matchobj->_match_state_save;
+    {
+      my $next_x_i = $$x_i;
+      my $next_p_i = $$p_i + 1;
+      $match_rest = $matchobj->_match_ARRAY_REST($x, $p, \$next_x_i, \$next_p_i);
+    }
+
+    if ( $match_rest ) {
+      $matched_rest = $match_rest;
+    } else {
+      # Restore match state if failed.
+      $matchobj->_match_state_restore($saved_state);      
+    }
+
+    # Did it work?
+    if ( $match_sub && $match_rest ) {
+      # Increment the subpattern match count.
+      ++ $count;
+      last TRY if ( defined $self->[1] && $count >= $self->[1] );
+    } else {
+      last TRY;
+    }
+  }
+
+  # If matched the correct number of things.
+  if ( $self->[0] <= $count ) {
+    $$p_i = $#$p;
+    $$x_i = $#$x;
+  } else {
+    $matched_rest = 0;
+  }
+
+  $matched_rest;
+}
+
+
+sub match
 {
   my ($self, $x, $matchobj) = @_;
 
-  ref($x) && $self->match_and($x, $matchobj);
+  confess "RE pattern must be used in ARRAY context";
+}
+
+
+##################################################
+
+package Data::Match::Pattern::QUES;
+
+our @ISA = qw(Data::Match::Pattern::RANG);
+
+sub new
+{
+  my ($self, @opts) = @_;
+  $self->SUPER::new(0, 1, @opts);
+}
+
+
+##################################################
+
+package Data::Match::Pattern::STAR;
+
+our @ISA = qw(Data::Match::Pattern::RANG);
+
+sub new
+{
+  my ($self, @opts) = @_;
+  $self->SUPER::new(0, undef, @opts);
+}
+
+
+##################################################
+
+package Data::Match::Pattern::PLUS;
+
+our @ISA = qw(Data::Match::Pattern::RANG);
+
+sub new
+{
+  my ($self, @opts) = @_;
+  $self->SUPER::new(1, undef, @opts);
 }
 
 
@@ -1385,18 +1655,14 @@ sub _match_each_ARRAY
 {
   my ($self, $x, $matchobj, $matches) = @_;
 
-  ++ $matchobj->{'depth'};
-  push(@{$matchobj->{'path'}}, 'ARRAY');
-  
   my $i = -1;
   for my $e ( @$x ) {
-    push(@{$matchobj->{'path'}}, ++ $i);
+    $matchobj->_match_path_push('ARRAY', ++ $i);
+
     ++ $$matches if $self->match_and($e, $matchobj);
-    pop(@{$matchobj->{'path'}});
+
+    $matchobj->_match_path_pop;
   }
-  
-  pop(@{$matchobj->{'path'}});
-  -- $matchobj->{'depth'};
 }
 
 
@@ -1404,14 +1670,11 @@ sub _match_each_HASH
 {
   my ($self, $x, $matchobj, $matches) = @_;
 
-  ++ $matchobj->{'depth'};
-  push(@{$matchobj->{'path'}}, 'HASH');
-  
   for my $k ( keys %$x ) {
     my @k = ( $k );
 
     # We compensate the path for hash slice.
-    push(@{$matchobj->{'path'}}, \@k);
+    $matchobj->_match_path_push('HASH', \@k);
     
     # Create a temporary hash slice.
     # because we are matching EACH element of the hash.
@@ -1424,11 +1687,8 @@ sub _match_each_HASH
 
     ++ $$matches if $self->match_and($slice, $matchobj);
     
-    pop(@{$matchobj->{'path'}});
+    $matchobj->_match_path_pop;
   }
-  
-  pop(@{$matchobj->{'path'}});
-  -- $matchobj->{'depth'};
 }
 
 
@@ -1436,14 +1696,11 @@ sub _match_each_SCALAR
 {
   my ($self, $x, $matchobj, $matches) = @_;
 
-  ++ $matchobj->{'depth'};
-  push(@{$matchobj->{'path'}}, 'SCALAR', undef);
+  $matchobj->_match_path_push('SCALAR', undef);
   
   ++ $$matches if $self->match_and($$x, $matchobj);
   
-  pop(@{$matchobj->{'path'}});
-  pop(@{$matchobj->{'path'}});
-  -- $matchobj->{'depth'};
+  $matchobj->_match_path_pop;
 }
 
 
@@ -1532,11 +1789,9 @@ sub _match_find_ARRAY
 
   my $i = -1;
   for my $e ( @$x ) {
-    push(@{$matchobj->{'path'}}, ++ $i);
-    
+    $matchobj->_match_path_push('ARRAY', ++ $i);
     $self->_match_find($e, $matchobj, $matches, $visited);
-    
-    pop(@{$matchobj->{'path'}});
+    $matchobj->_match_path_pop;
   }
 }
 
@@ -1546,19 +1801,14 @@ sub _match_find_HASH
   my ($self, $x, $matchobj, $matches, $visited) = @_;
 
   for my $k ( keys %$x ) {
-    
-    push(@{$matchobj->{'path'}}, [ $k ]);
-    
+    $matchobj->_match_path_push('HASH', [ $k ]);
     # This needs a new Slice class.
     $self->_match_find($k, $matchobj, $matches); # HUH?
+    $matchobj->_match_path_pop;
     
-    pop(@{$matchobj->{'path'}});
-    
-    push(@{$matchobj->{'path'}}, $k);
-    
+    $matchobj->_match_path_push('HASH', $k);
     $self->_match_find($x->{$k}, $matchobj, $matches, $visited);
-    
-    pop(@{$matchobj->{'path'}});
+    $matchobj->_match_path_pop;
   }
 }
 
@@ -1587,32 +1837,15 @@ sub _match_find
       $visit->($x, $visitor, $matchobj);
     }
     elsif ( UNIVERSAL::isa($x, 'ARRAY') ) {
-      ++ $matchobj->{'depth'};
-      push(@{$matchobj->{'path'}}, 'ARRAY');
-
       $self->_match_find_ARRAY($x, $matchobj, $matches, $visited);
-
-      pop(@{$matchobj->{'path'}});
-      -- $matchobj->{'depth'};
     }
     elsif ( UNIVERSAL::isa($x, 'HASH') ) {
-      ++ $matchobj->{'depth'};
-      push(@{$matchobj->{'path'}}, 'HASH');
-
       $self->_match_find_HASH($x, $matchobj, $matches, $visited);
-
-      pop(@{$matchobj->{'path'}});
-      -- $matchobj->{'depth'};
     }
     elsif ( UNIVERSAL::isa($x, 'SCALAR') ) {
-      ++ $matchobj->{'depth'};
-      push(@{$matchobj->{'path'}}, 'SCALAR', undef);
-
+      $matchobj->_match_path_push('SCALAR', undef);
       $self->_match_find($$x, $matchobj, $matches, $visited);
-
-      pop(@{$matchobj->{'path'}});
-      pop(@{$matchobj->{'path'}});
-      -- $matchobj->{'depth'};
+      $matchobj->_match_path_pop;
     }
     else {
       warn "Huh?";
@@ -1649,6 +1882,11 @@ sub new
 }
 
 
+sub _SLICE_SRC { $_[0][0]; }
+sub _SLICE_BEG { $_[0][1]; }
+sub _SLICE_END { $_[0][2]; }
+
+
 sub TIEARRAY
 {
   my ($cls, $src, $from, $to) = @_;
@@ -1817,15 +2055,15 @@ sub NEXTKEY
 
 =head1 VERSION
 
-Version 0.03, $Revision: 1.8 $.
+Version 0.05, $Revision: 1.11 $.
 
 =head1 AUTHOR
 
-Kurt A. Stephens <kurtstephens@acm.org>
+Kurt A. Stephens <ks.perl@kurtstephens.com>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001, Kurt A. Stephens and ION, INC.
+Copyright (c) 2001, 2002 Kurt A. Stephens and ION, INC.
 
 =head1 SEE ALSO
 
@@ -1836,3 +2074,12 @@ L<perl>, L<Array::PatternMatcher>, L<Data::Compare>, L<Data::Dumper>, L<Data::DR
 ##################################################
 
 1;
+
+### Keep these comments at end of file: kstephens@cpan.org 2001/12/28 ###
+### Local Variables: ###
+### mode:perl ###
+### perl-indent-level:2 ###
+### perl-continued-statement-offset:0 ###
+### perl-brace-offset:0 ###
+### perl-label-offset:0 ###
+### End: ###
